//gD[_OpGtFNgt@CB
//SDKTvGlowM

// transforms
float4x4 WorldView  : WORLDVIEW;
float4x4 View : VIEW; 
float4x4 Projection : PROJECTION;
float4x4 World : WORLD;

// light direction (view space)
float3 LightDir < string UIDirectional = "Light Direction"; > = normalize(float3(0.0f, 0.0f, 1.0f));
// glow parameters
float4 EdgeColor = float4(0.0f,0.0f,0.0f,1.0f);
float4 Diffuse2 = float4(1.0f,1.0f,1.0f,1.0f);
float4 Diffuse =  float4(1.0f,1.0f,1.0f,1.0f);
float  GlowThickness = 0.005f;
float Fogend;
float Fogcoef;
int Shadow;
struct VSTOON_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse : COLOR;
    float Fog : FOG;
    float2 TexCoord0 : TEXCOORD0;
   float2 TexCoord1 : TEXCOORD1;
};

VSTOON_OUTPUT VSToon(float4 Position : POSITION
                                        ,float3 Normal   : NORMAL,
                                       float2 TexCoord0 : TEXCOORD0)
{
   VSTOON_OUTPUT Out = (VSTOON_OUTPUT)0;
  float3 P = mul(Position, WorldView);
  float3 N = normalize(mul(Normal,World));
  float3 L = -normalize(LightDir);
  Out.Position = mul(float4(P, 1), Projection);
  Out.TexCoord0 = TexCoord0;
  Out.TexCoord1.x = 0.1f;
  if(Shadow)Out.TexCoord1.y = 0.9f;
  else Out.TexCoord1.y = (dot(N, L)+1.0f)/2;

  Out.Fog = min(max((-P.z+Fogend)*Fogcoef,0.0f),1.0f);
  Out.Diffuse = Diffuse*Diffuse2;
  return Out;
}

struct VSTOONNOTEX_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse : COLOR;
    float Fog : FOG;
    float2 TexCoord0 : TEXCOORD0;
};

VSTOON_OUTPUT VSToonNoTex(float4 Position : POSITION
                                        ,float3 Normal   : NORMAL
                                       )
{
   VSTOON_OUTPUT Out = (VSTOON_OUTPUT)0;
  float3 P = mul(Position, WorldView);
  float3 N = normalize(mul(Normal,World));
  float3 L = -normalize(LightDir);
  Out.Position = mul(float4(P, 1), Projection);
  Out.TexCoord0.x = 0.1f;
  if(Shadow)Out.TexCoord0.y = 0.9f;
  else Out.TexCoord0.y = (dot(N, L)+1.0f)/2;

  Out.Fog = min(max((-P.z+Fogend)*Fogcoef,0.0f),1.0f);
  Out.Diffuse = Diffuse*Diffuse2;
  return Out;
}

struct VSEDGE_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse  : COLOR;
     float Fog : FOG;
};

// draws a transparent hull of the unskinned object.
VSEDGE_OUTPUT VSEdge
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL
    )
{
    VSEDGE_OUTPUT Out = (VSEDGE_OUTPUT)0;

    float3 N = normalize(mul(Normal, (float3x3)WorldView));     // normal (view space)
    float3 P = mul(Position, WorldView);    // displaced position (view space)
    float3 A = float3(0, 0, 1);                                 // glow axis
    float4 T = mul(N,Projection);
    T.z = 0.0f;
    T.w = 0.0f;
    T = normalize(T);
    Out.Position = mul(float4(P,1),Projection);// projected position
    Out.Position+= T*GlowThickness*Out.Position.w;
    Out.Diffuse  = EdgeColor; // modulated glow color + glow ambient
    Out.Fog = min(max((-P.z+Fogend)*Fogcoef,0.0f),1.0f);
    return Out;    
}

technique TToon
{
 pass PToon
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSToon();
        PixelShader  = NULL;
        
        // texture
        //Texture[0] = (Tex0);
        ColorOp[0]   =MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        ColorOp[1]   =MODULATE;
        ColorArg1[1] = TEXTURE;
        ColorArg2[1] = CURRENT;


        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = POINT;

        // set up texture stage states for single texture modulated by diffuse 
        AlphaBlendEnable = TRUE;
        AlphaTestEnable = False;
        Lighting = False;
    }
    pass PEdge
    {   
        // glow shader
        VertexShader = compile vs_1_1 VSEdge();
        PixelShader  = NULL;
        
        // no texture
        Texture[0] = NULL;

        // set up texture stage states to use the diffuse color
        ColorOp[0]   = SELECTARG2;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = SELECTARG2;
        AlphaArg2[0] = DIFFUSE;
        CullMode = CW;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;
        AlphaBlendEnable = False;
           AlphaTestEnable = False;

   }
}
 
technique TToonNoTex
{
 pass PToon
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSToonNoTex();
        PixelShader  = NULL;
        
        // texture
        //Texture[0] = (Tex0);
        ColorOp[0]   =MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
    
        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = POINT;

        // set up texture stage states for single texture modulated by diffuse 
        AlphaBlendEnable = TRUE;
        AlphaTestEnable = False;
        Lighting = False;
    }
    pass PEdge
    {   
        // glow shader
        VertexShader = compile vs_1_1 VSEdge();
        PixelShader  = NULL;
        
        // no texture
        Texture[0] = NULL;

        // set up texture stage states to use the diffuse color
        ColorOp[0]   = SELECTARG2;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = SELECTARG2;
        AlphaArg2[0] = DIFFUSE;
        CullMode = CW;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;
        AlphaBlendEnable = False;
           AlphaTestEnable = False;

   }
}
